/***
*
*	Copyright (c) 1996-2001, Valve LLC. All rights reserved.
*	
*	This product contains software technology licensed from Id 
*	Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*	All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/
#if !defined( OEM_BUILD )

#include "extdll.h"
#include "util.h"
#include "cbase.h"
#include "monsters.h"
#include "weapons.h"
#include "nodes.h"
#include "player.h"
#include "gamerules.h"

#include "customentity.h"
#include "schedule.h"




enum rpg_e {
	RPG_IDLE = 0,
	RPG_FIDGET,
	RPG_RELOAD,		// to reload
	RPG_FIRE2,		// to empty
	RPG_HOLSTER1,	// loaded
	RPG_DRAW1,		// loaded
	RPG_HOLSTER2,	// unloaded
	RPG_DRAW_UL,	// unloaded
	RPG_IDLE_UL,	// unloaded idle
	RPG_FIDGET_UL,	// unloaded fidget
};

LINK_ENTITY_TO_CLASS( weapon_rpg, CRpg );

#ifndef CLIENT_DLL

LINK_ENTITY_TO_CLASS( laser_spot, CLaserSpot );

//=========================================================
//=========================================================
CLaserSpot *CLaserSpot::CreateSpot( void )
{
	CLaserSpot *pSpot = GetClassPtr( (CLaserSpot *)NULL );
	pSpot->Spawn();

	pSpot->pev->classname = MAKE_STRING("laser_spot");

	return pSpot;
}

//=========================================================
//=========================================================
void CLaserSpot::Spawn( void )
{
	Precache( );
	pev->movetype = MOVETYPE_NONE;
	pev->solid = SOLID_NOT;

        pev->rendermode = kRenderTransAdd;
//        pev->renderfx = kRenderFxNoDissipation;
        pev->renderamt = 255;


        SET_MODEL(ENT(pev), "models/select.mdl");
        pev->skin = 2;
	UTIL_SetOrigin( pev, pev->origin );
};

//=========================================================
// Suspend- make the laser sight invisible. 
//=========================================================
void CLaserSpot::Suspend( float flSuspendTime )
{
	pev->effects |= EF_NODRAW;
	
	SetThink( &CLaserSpot::Revive );
	pev->nextthink = gpGlobals->time + flSuspendTime;
}

//=========================================================
// Revive - bring a suspended laser sight back.
//=========================================================
void CLaserSpot::Revive( void )
{
	pev->effects &= ~EF_NODRAW;

	SetThink( NULL );
}

void CLaserSpot::Precache( void )
{
        //PRECACHE_MODEL("models/select.mdl");

};

LINK_ENTITY_TO_CLASS( rpg_rocket, CRpgRocket );

//=========================================================
//=========================================================
CRpgRocket *CRpgRocket::CreateRpgRocket( Vector vecOrigin, Vector vecAngles, CBaseEntity *pOwner, CRpg *pLauncher )
{
	CRpgRocket *pRocket = GetClassPtr( (CRpgRocket *)NULL );

	UTIL_SetOrigin( pRocket->pev, vecOrigin );
	pRocket->pev->angles = vecAngles;
	pRocket->Spawn();
	pRocket->SetTouch( &CRpgRocket::RocketTouch );
	pRocket->m_pLauncher = pLauncher;// remember what RPG fired me. 
	pRocket->m_pLauncher->m_cActiveRockets++;// register this missile as active for the launcher
	pRocket->pev->owner = pOwner->edict();

	return pRocket;
}

//=========================================================
//=========================================================
void CRpgRocket :: Spawn( void )
{
	Precache( );
	// motor
	pev->movetype = MOVETYPE_BOUNCE;
	pev->solid = SOLID_BBOX;

	SET_MODEL(ENT(pev), "models/rpgrocket.mdl");
	UTIL_SetSize(pev, Vector( 0, 0, 0), Vector(0, 0, 0));
	UTIL_SetOrigin( pev, pev->origin );

	pev->classname = MAKE_STRING("rpg_rocket");

	SetThink( &CRpgRocket::IgniteThink );
	SetTouch( &CRpgRocket::ExplodeTouch );

	pev->angles.x -= 30;
	UTIL_MakeVectors( pev->angles );
	pev->angles.x = -(pev->angles.x + 30);

	pev->velocity = gpGlobals->v_forward * 250;
	pev->gravity = 0.5;

	pev->nextthink = gpGlobals->time + 0.4;

	pev->dmg = gSkillData.plrDmgRPG;
}

//=========================================================
//=========================================================
void CRpgRocket :: RocketTouch ( CBaseEntity *pOther )
{
	if ( m_pLauncher )
	{
                // my launcher is still around, tell it I'm dead.
		m_pLauncher->m_cActiveRockets--;
	}

	STOP_SOUND( edict(), CHAN_VOICE, "weapons/rocket1.wav" );
	ExplodeTouch( pOther );
}

//=========================================================
//=========================================================
void CRpgRocket :: Precache( void )
{
	PRECACHE_MODEL("models/rpgrocket.mdl");
	m_iTrail = PRECACHE_MODEL("sprites/smoke.spr");
	PRECACHE_SOUND ("weapons/rocket1.wav");

}


void CRpgRocket :: IgniteThink( void  )
{
	// pev->movetype = MOVETYPE_TOSS;

	pev->movetype = MOVETYPE_FLY;
	pev->effects |= EF_LIGHT;

	// make rocket sound
	EMIT_SOUND( ENT(pev), CHAN_VOICE, "weapons/rocket1.wav", 1, 0.5 );

	// rocket trail
	MESSAGE_BEGIN( MSG_BROADCAST, SVC_TEMPENTITY );

		WRITE_BYTE( TE_BEAMFOLLOW );
		WRITE_SHORT(entindex());	// entity
		WRITE_SHORT(m_iTrail );	// model
		WRITE_BYTE( 40 ); // life
		WRITE_BYTE( 5 );  // width
		WRITE_BYTE( 224 );   // r, g, b
		WRITE_BYTE( 224 );   // r, g, b
		WRITE_BYTE( 255 );   // r, g, b
		WRITE_BYTE( 255 );	// brightness

	MESSAGE_END();  // move PHS/PVS data sending into here (SEND_ALL, SEND_PVS, SEND_PHS)

	m_flIgniteTime = gpGlobals->time;

	// set to follow laser spot
	SetThink( &CRpgRocket::FollowThink );
	pev->nextthink = gpGlobals->time + 0.1;
}


void CRpgRocket :: FollowThink( void  )
{
	CBaseEntity *pOther = NULL;
	Vector vecTarget;
	Vector vecDir;
	float flDist, flMax, flDot;
	TraceResult tr;

	UTIL_MakeAimVectors( pev->angles );

	vecTarget = gpGlobals->v_forward;
	flMax = 4096;
	
	// Examine all entities within a reasonable radius
	while ((pOther = UTIL_FindEntityByClassname( pOther, "laser_spot" )) != NULL)
	{
		UTIL_TraceLine ( pev->origin, pOther->pev->origin, dont_ignore_monsters, ENT(pev), &tr );
		// ALERT( at_console, "%f\n", tr.flFraction );
		if (tr.flFraction >= 0.90)
		{
			vecDir = pOther->pev->origin - pev->origin;
			flDist = vecDir.Length( );
			vecDir = vecDir.Normalize( );
			flDot = DotProduct( gpGlobals->v_forward, vecDir );
			if ((flDot > 0) && (flDist * (1 - flDot) < flMax))
			{
				flMax = flDist * (1 - flDot);
				vecTarget = vecDir;
			}
		}
	}

	pev->angles = UTIL_VecToAngles( vecTarget );

	// this acceleration and turning math is totally wrong, but it seems to respond well so don't change it.
	float flSpeed = pev->velocity.Length();
	if (gpGlobals->time - m_flIgniteTime < 1.0)
	{
		pev->velocity = pev->velocity * 0.2 + vecTarget * (flSpeed * 0.8 + 400);
		if (pev->waterlevel == 3)
		{
			// go slow underwater
			if (pev->velocity.Length() > 300)
			{
				pev->velocity = pev->velocity.Normalize() * 300;
			}
			UTIL_BubbleTrail( pev->origin - pev->velocity * 0.1, pev->origin, 4 );
		} 
		else 
		{
			if (pev->velocity.Length() > 2000)
			{
				pev->velocity = pev->velocity.Normalize() * 2000;
			}
		}
	}
	else
	{
		if (pev->effects & EF_LIGHT)
		{
			pev->effects = 0;
			STOP_SOUND( ENT(pev), CHAN_VOICE, "weapons/rocket1.wav" );
		}
		pev->velocity = pev->velocity * 0.2 + vecTarget * flSpeed * 0.798;
		if (pev->waterlevel == 0 && pev->velocity.Length() < 1500)
		{
			Detonate( );
		}
	}
	// ALERT( at_console, "%.0f\n", flSpeed );

	pev->nextthink = gpGlobals->time + 0.1;
}
#endif



void CRpg::Reload( void )
{
        // end turn.

        if ( m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] > 0 )
                return;


}

void CRpg::Spawn( )
{
	Precache( );
	m_iId = WEAPON_RPG;

	SET_MODEL(ENT(pev), "models/w_rpg.mdl");
	m_fSpotActive = 1;

        m_iDefaultAmmo = RPG_DEFAULT_GIVE;


	FallInit();// get ready to fall down.
}


void CRpg::Precache( void )
{
	PRECACHE_MODEL("models/w_rpg.mdl");
        PRECACHE_MODEL("models/null.mdl");
        PRECACHE_MODEL("sprites/arrow.spr");//this is for the move-beam.
        PRECACHE_SOUND ("rifle.wav");

        m_iShell = PRECACHE_MODEL ("models/shotgunshell.mdl");// shotgun shell

        //UTIL_PrecacheOther( "laser_spot" );

        //m_usRpg = PRECACHE_EVENT ( 1, "events/rpg.sc" );


        UTIL_PrecacheOther( "monster_zombie" );

}


int CRpg::GetItemInfo(ItemInfo *p)
{
	p->pszName = STRING(pev->classname);
	p->pszAmmo1 = "rockets";
	p->iMaxAmmo1 = ROCKET_MAX_CARRY;
	p->pszAmmo2 = NULL;
	p->iMaxAmmo2 = -1;
        //p->iMaxClip = RPG_MAX_CLIP;
        p->iMaxClip = WEAPON_NOCLIP;
	p->iSlot = 3;
	p->iPosition = 0;
	p->iId = m_iId = WEAPON_RPG;
	p->iFlags = 0;
	p->iWeight = RPG_WEIGHT;

	return 1;
}

int CRpg::AddToPlayer( CBasePlayer *pPlayer )
{
	if ( CBasePlayerWeapon::AddToPlayer( pPlayer ) )
	{
		MESSAGE_BEGIN( MSG_ONE, gmsgWeapPickup, NULL, pPlayer->pev );
			WRITE_BYTE( m_iId );
		MESSAGE_END();
		return TRUE;
	}
	return FALSE;
}

BOOL CRpg::Deploy( )
{
        return DefaultDeploy( "models/null.mdl", "models/null.mdl", RPG_DRAW1, "rpg" );
}


BOOL CRpg::CanHolster( void )
{
	if ( m_fSpotActive && m_cActiveRockets )
	{
		// can't put away while guiding a missile.
		return FALSE;
	}

	return TRUE;
}

void CRpg::Holster( int skiplocal /* = 0 */ )
{
	m_fInReload = FALSE;// cancel any reload in progress.

	m_pPlayer->m_flNextAttack = UTIL_WeaponTimeBase() + 0.5;

#ifndef CLIENT_DLL
	if (m_pSpot)
	{
		m_pSpot->Killed( NULL, GIB_NEVER );
		m_pSpot = NULL;
	}
#endif
}



void CRpg::PrimaryAttack()
{

        //I have ammo. that means: critter placement
        if ( m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] > 0 )
        {


                //this code makes sure we are not stacking units.
                //copied from monstermaker.
                Vector mins, maxs;
                #ifndef CLIENT_DLL
                mins = m_pSpot->pev->origin - Vector( 20, 20, 0 );
                maxs = m_pSpot->pev->origin + Vector( 20, 20, 64 );
                #endif
                int count;

                CBaseEntity *pList[2];


                #ifndef CLIENT_DLL
                count = UTIL_EntitiesInBox( pList, 2, mins, maxs, FL_MONSTER );
                #endif

                if ( count )
                {
                        // don't build a stack of monsters!
                        //ALERT(at_console,"deploy block!\n");
                        EMIT_SOUND( ENT(pev), CHAN_ITEM, "common/wpn_denyselect.wav", 1, ATTN_NORM);
                }
                else
                {

                        TraceResult tr;
                        edict_t *pWorld = g_engfuncs.pfnPEntityOfEntIndex( 0 );
                        //edict_t *pWorld = INDEXENT(0);
                        

                        const char *pTextureName;

                        #ifndef CLIENT_DLL
                        UTIL_TraceLine( m_pSpot->pev->origin + Vector(0,0,16), m_pSpot->pev->origin + Vector(0,0,-32), ignore_monsters, edict(), &tr);
                        

                        if ( tr.pHit )
                               pWorld = tr.pHit;

                        
                        if (pWorld)
                                pTextureName = TRACE_TEXTURE( pWorld, m_pSpot->pev->origin + Vector(0,0,16), m_pSpot->pev->origin + Vector(0,0,-32) );
                        #endif
                        
                        //const char *pTeamName;
                        //pTeamName = m_pPlayer->TeamID();



                        if (pTextureName && m_pPlayer->TeamID())
                        {
                                if      (
                                        ((stricmp( pTextureName, "generic015t") == 0) && (FStrEq(m_pPlayer->TeamID(), "Red")))
                                        ||
                                        ((stricmp( pTextureName, "generic015v2") == 0) && (FStrEq(m_pPlayer->TeamID(), "Blue")))
                                        )
                                {
                                        //monster placement.
                                        #ifndef CLIENT_DLL
                                        //bc
                                        //CBaseEntity *pSqueak = CBaseEntity::Create( "monster_zombie", m_pSpot->pev->origin + Vector(0,0,1), Vector(0,0,0), m_pPlayer->edict() );
                                        CBaseEntity::Create( "monster_zombie", m_pSpot->pev->origin + Vector(0,0,1), Vector(0,0,0), m_pPlayer->edict() );
                                        #endif
                        
                                        //poit!!
                                        if ( m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType] > 1 )
                                                EMIT_SOUND( ENT(pev), CHAN_ITEM, "common/wpn_hudoff.wav", 1, ATTN_NORM);
                                        else
                                                EMIT_SOUND( ENT(pev), CHAN_ITEM, "common/wpn_hudon.wav", 1, ATTN_NORM);


                
                                        //decrement the remaining unit amount.
                                        m_pPlayer->m_rgAmmo[m_iPrimaryAmmoType]--;
                                }
                                else
                                {
                                        EMIT_SOUND( ENT(pev), CHAN_ITEM, "common/wpn_denyselect.wav", 1, ATTN_NORM);
                                }
                        }
                }
        }
        else
        {

                //we are no longer in unit deployment mode,
                //we are now interacting with world.

                //UTIL_MakeVectors(m_pPlayer->pev->v_angle);
                //TraceResult tr;
                //UTIL_TraceLine( pev->origin, pev->origin + (gpGlobals->v_forward * 4096), dont_ignore_monsters, edict(), &tr );

                CBaseEntity  *pEntity = NULL;
                //pEntity = CBaseEntity::Instance(tr.pHit);




                Vector mins, maxs;
                #ifndef CLIENT_DLL
                mins = m_pSpot->pev->origin - Vector( 20, 20, 1 );
                maxs = m_pSpot->pev->origin + Vector( 20, 20, 64 );
                #endif
                int count;

                CBaseEntity *pList[2];

                #ifndef CLIENT_DLL
                count = UTIL_EntitiesInBox( pList, 4, mins, maxs, FL_MONSTER );
                #endif

                if ( count )
                {
                        for ( int i = 0; i < count; i++ )
                        {
                                //SELECT THIS UNIT. make it our pENTITY
                                if ( pList[i] && pList[ i ]->pev->deadflag == DEAD_NO )
                                        pEntity = CBaseEntity::Instance(pList[i]->pev);
                        }
                }


                //if ( tr.pHit && Instance( tr.pHit )->pev->solid != SOLID_BSP) 
                {
                        //CBaseEntity  *pEntity = NULL;
                        //pEntity = CBaseEntity::Instance(tr.pHit);


                        //okay, we have hit an entity. deal with entity.
                        if (pEntity)
                        {
                                CBaseMonster *pTarget = NULL;

                                //this is the unit in question.
                                pTarget = pEntity->MyMonsterPointer();


                                //check if we are owner of this unit.
                                //second line is the singleplayer HACK

                                if (( pTarget && (pTarget->pev->owner == m_pPlayer->edict()) )  ||
                                    ( pTarget && (pTarget->pev->owner == NULL) && ( FClassnameIs( pTarget->pev, "monster_zombie" ) ))
                                    )
                                {
                                        if (pTarget->pev->owner == NULL)
                                            pTarget->pev->owner = m_pPlayer->edict();//HACKHACKHACK for singleplaeyr

                                        //we own this unit

                                        //already active!.
                                        if (pTarget->IsActive == 1)
                                                return;
                                        else
                                        {
                                                //select unit.

                                                //remember who is the current unit.
                                                CBaseMonster *pCurrent = pTarget;
                                                CBaseEntity *pCurrentEntity = pEntity;

                                                do
                                                {
                                                        //increment to next unit
                                                        #ifndef CLIENT_DLL
                                                        pEntity = UTIL_FindEntityByClassname( pEntity, "monster_zombie" );
                                                        #endif


                                                        if (pEntity)
                                                        {
                                                                //check if we own the unit.
                                                                if (pEntity->pev->owner == m_pPlayer->edict())
                                                                {
                                                                        //we own this unit.
                                                                        pTarget = pEntity->MyMonsterPointer();
                                                                        if (pTarget)
                                                                        {
                                                                                //deactivate this unit.
                                                                                if (pTarget->IsActive == 1)
                                                                                        pTarget->IsActive = 0;
                                                                        }
                                                                }
                                                        }
                                                        
                                                }
                                                while ( pEntity != pCurrentEntity );//loop

                                                //activate the unit.
                                                pCurrent->IsActive = 1;
                                                EMIT_SOUND( ENT(pev), CHAN_ITEM, "common/wpn_select.wav", 1, ATTN_NORM);
                                        }
                                }
                                else
                                {
                                        //we DO NOT own this unit.

                                        EMIT_SOUND( ENT(pev), CHAN_ITEM, "common/wpn_denyselect.wav", 1, ATTN_NORM);
                                }


                        }
                }




        }

        m_flNextPrimaryAttack = UTIL_WeaponTimeBase() + 0.2;
}


void CRpg::SecondaryAttack()
{
        CBaseEntity *pEntity = NULL;
        CBaseMonster *pTarget = NULL;

        //pick any random unit
        #ifndef CLIENT_DLL
        pEntity = UTIL_FindEntityByClassname( pEntity, "monster_zombie" );
        #endif

        if (pEntity)
        {
                do
                {
                        //check if we own the unit.
                        if (pEntity->pev->owner == m_pPlayer->edict())
                        {
                                pTarget = pEntity->MyMonsterPointer();
                                if (pTarget->IsActive == 1)
                                {
                                        //bc
                                        Vector mins, maxs;
                                        #ifndef CLIENT_DLL
                                        mins = m_pSpot->pev->origin - Vector( 20, 20, 0 );
                                        maxs = m_pSpot->pev->origin + Vector( 20, 20, 64 );
                                        #endif
                                        int count = NULL;
                        
                                        CBaseEntity *pList[2];
                        
                                        #ifndef CLIENT_DLL
                                        count = UTIL_EntitiesInBox( pList, 4, mins, maxs, FL_MONSTER );
                                        #endif

                                        if ( count )
                                        {
                                                //found a critter on our selector.

                                                CBaseEntity *pEntity = NULL;

                                                for ( int i = 0; i < count; i++ )
                                                {
                                                        if ( pList[i] && pList[ i ]->pev->deadflag == DEAD_NO )
                                                                pEntity = CBaseEntity::Instance(pList[i]->pev);
                                                }
                        
                                                if (pEntity)
                                                {
                                                        //this is the unit in question.

                                                        //check if we are owner of this unit.
                                                        //BCTEMP
                                                        if ( pEntity->pev->owner != m_pPlayer->edict() )
                                                        {
                                                                //holy bejesus
                                                                //player has targetted monster!!!!

                                                                //check if playerunit has LOS
                                                                TraceResult tr;
                                                                UTIL_TraceLine ( pTarget->Center(), pEntity->Center(), dont_ignore_monsters, pTarget->edict(), &tr );



                                                                if ( tr.flFraction == 1.0 || tr.pHit == pEntity->edict() )
                                                                {
                                                                        //holy cow we have LOS!!
                                                                        //ALERT(at_console,"BANGG!!!\n");
                                                                        //pTarget->pev->effects = EF_MUZZLEFLASH;
                                                                        pEntity->TakeDamage(pTarget->pev,pTarget->pev,gSkillData.plrDmgRPG,DMG_NEVERGIB);

                                                                        //face target.
                                                                        #ifndef CLIENT_DLL
                                                                        pTarget->pev->angles.y = UTIL_VecToYaw( pEntity->pev->origin - pTarget->pev->origin );
                                                                        #endif

                                                                        //bang sound
                                                                        EMIT_SOUND_DYN( edict(), CHAN_VOICE, "rifle.wav", 1, ATTN_IDLE, 0, PITCH_NORM );

                                                                        //shell eject.
                                                                        UTIL_MakeVectors( pTarget->pev->v_angle);
                                                                        EjectBrass ( pTarget->pev->origin + gpGlobals->v_up * 32 + gpGlobals->v_forward * 32, gpGlobals->v_up * RANDOM_LONG(312,348) + gpGlobals->v_right * RANDOM_LONG(16,48), pTarget->pev->angles.y, m_iShell, TE_BOUNCE_SHELL );
                                                                        EjectBrass ( pTarget->pev->origin + gpGlobals->v_up * 32 + gpGlobals->v_forward * 32, gpGlobals->v_up * RANDOM_LONG(348,384) + gpGlobals->v_right * RANDOM_LONG(16,48), pTarget->pev->angles.y, m_iShell, TE_BOUNCE_SHELL );
                                                                        EjectBrass ( pTarget->pev->origin + gpGlobals->v_up * 32 + gpGlobals->v_forward * 32, gpGlobals->v_up * RANDOM_LONG(384,420) + gpGlobals->v_right * RANDOM_LONG(16,48), pTarget->pev->angles.y, m_iShell, TE_BOUNCE_SHELL );

                                                                        //steambubble fx.
                                                                        #ifndef CLIENT_DLL
                                                                        UTIL_BubbleTrail( pTarget->Center(), pEntity->Center(), 16 );

                                                                        #endif
                                                                                                                                                
                                                                }
                                                                else
                                                                {
                                                                        UTIL_ClientPrintAll( HUD_PRINTCENTER, "! NO LINE-OF-SIGHT !\n");
                                                                        //ClientPrint( &pEntity->v,HUD_PRINTCENTER, "! NO LINE-OF-SIGHT !\n");
                                                                        //ClientPrint( ,HUD_PRINTCENTER, "! NO LINE-OF-SIGHT !\n");
                                                                }
                                                                        //ALERT(at_console,"NO LOS\n");
                                                        }
                                                }
                                        }
                                        else
                                        {

                                        //okay, player just clicked on the ground.
                                        //that means: player wants to move the unit.

                                        //pTarget->m_vecMoveGoal = m_pSpot->pev->origin;
                                        //pTarget->m_vecMoveGoal = Vector(128,0,0);

                                        EMIT_SOUND( ENT(pev), CHAN_ITEM, "common/wpn_moveselect.wav", 1, ATTN_NORM);

                                        #ifndef CLIENT_DLL//bc gee messy
                                        pTarget->BuildRoute( m_pSpot->pev->origin, bits_MF_TO_TARGETENT, m_pSpot );

                                        //ARRRRRRR
                                        //pTarget->m_hTargetEnt = m_pSpot;
                                        //pTarget->m_vecMoveGoal = m_pSpot->pev->origin;
                                        //pTarget->MakeIdealYaw ( m_pSpot->pev->origin );
                                        //pTarget->BuildRoute( m_pSpot->pev->origin, bits_MF_TO_LOCATION, NULL );
                                        //pTarget->BuildRoute( m_pSpot->pev->origin, bits_MF_TO_TARGETENT, m_pSpot );
                                        //UTIL_MoveToOrigin ( ENT(pEntity->pev), m_Route[ m_iRouteIndex ].vecLocation, flStep, MOVE_NORMAL );
                                        //WALK_MOVE( ENT(pEntity->pev), 200, pTarget->m_flGroundSpeed, WALKMOVE_NORMAL );
                                        //WALK_MOVE( ENT(pev), pev->ideal_yaw, m_flGroundSpeed * flInterval, WALKMOVE_NORMAL );
                                        //pTarget->pev->angles = UTIL_VecToAngles( m_pSpot->pev->origin - pTarget->pev->origin );
                                        //WALK_MOVE( pTarget->edict(), pTarget->pev->angles.y, 64, WALKMOVE_NORMAL );
                                        #endif

                                        //CBeam *pBeam;

                                        if (pBeam)
                                          UTIL_Remove(pBeam);

                                        pBeam = CBeam::BeamCreate( "sprites/arrow.spr", 16 );
                                        pBeam->PointEntInit( m_pSpot->pev->origin, pTarget->entindex());
                                        //pBeam->SetFlags( BEAM_FSOLID );
                                        //pBeam->SetColor( 255, 255, 255 );
                                        pBeam->SetScrollRate( 10 );
                                        pBeam->SetThink( &CRpg::SUB_Remove );
                                        pBeam->pev->nextthink = gpGlobals->time + 4;
                                        }
                                }
                        }
        
                        
                        //increment to next unit
                        #ifndef CLIENT_DLL
                        pEntity = UTIL_FindEntityByClassname( pEntity, "monster_zombie" );
                        #endif
                }
                while ( pEntity != NULL );//loop
        }




        m_flNextSecondaryAttack = UTIL_WeaponTimeBase() + 0.3;
}


void CRpg::WeaponIdle( void )
{
	UpdateSpot( );

}



void CRpg::UpdateSpot( void )
{

#ifndef CLIENT_DLL
	if (m_fSpotActive)
	{
		if (!m_pSpot)
		{
			m_pSpot = CLaserSpot::CreateSpot();
		}

		UTIL_MakeVectors( m_pPlayer->pev->v_angle );
                Vector vecSrc = m_pPlayer->GetGunPosition( );
		Vector vecAiming = gpGlobals->v_forward;

		TraceResult tr;

                UTIL_TraceLine ( vecSrc, vecSrc + vecAiming * 8192, ignore_monsters, ENT(m_pPlayer->pev), &tr );
		
     // code for aligning the selectbox to a locked "multiples of 64" grid.
     //ALERT(at_console,"mod %i\n", (int(tr.vecEndPos.y / 64)) * 64     );

                int remainder;
        
                //x coordinate
                remainder = tr.vecEndPos.x;
                remainder = remainder % 64;
                if (tr.vecEndPos.x > 0)
                        tr.vecEndPos.x +=32;
                else
                        tr.vecEndPos.x -=32;
                tr.vecEndPos.x = int(tr.vecEndPos.x) - remainder;

        
                //y coordinate
                remainder = tr.vecEndPos.y;
                remainder = remainder % 64;
                if (tr.vecEndPos.y > 0)
                        tr.vecEndPos.y +=32;
                else
                        tr.vecEndPos.y -=32;
                tr.vecEndPos.y = int(tr.vecEndPos.y) - remainder;

                TraceResult tz;
                UTIL_TraceLine ( tr.vecEndPos, Vector(0,0,-8192), ignore_monsters, ENT(m_pPlayer->pev), &tz );
                tr.vecEndPos.z = tz.vecEndPos.z;


                //place the selectbox into world.
                UTIL_SetOrigin( m_pSpot->pev, tr.vecEndPos );

                //bc
                //crosshairs.

                Vector mins, maxs;
                #ifndef CLIENT_DLL
                mins = m_pSpot->pev->origin - Vector( 20, 20, 0 );
                maxs = m_pSpot->pev->origin + Vector( 20, 20, 64 );
                #endif
                int count;

                CBaseEntity *pList[2];
                CBaseEntity *pEntity = NULL;

                #ifndef CLIENT_DLL
                count = UTIL_EntitiesInBox( pList, 4, mins, maxs, FL_MONSTER );
                #endif
                if ( count )
                {
                        for ( int i = 0; i < count; i++ )
                        {
                                if ( pList[i] && pList[ i ]->pev->deadflag == DEAD_NO )
                                {
                                        pEntity = CBaseEntity::Instance(pList[i]->pev);
                                        if (pEntity)
                                        {
                                                //on our team?
                                                if ( pEntity->pev->owner != m_pPlayer->edict() )
                                                {
                                                        //enemy!!
                                                        //ALERT(at_console,"YAR!\n");
                                                }
                                        }
                                }
                        }
                }

	}
#endif

}


class CRpgAmmo : public CBasePlayerAmmo
{
	void Spawn( void )
	{ 
		Precache( );
		SET_MODEL(ENT(pev), "models/w_rpgammo.mdl");
		CBasePlayerAmmo::Spawn( );
	}
	void Precache( void )
	{
		PRECACHE_MODEL ("models/w_rpgammo.mdl");
		PRECACHE_SOUND("items/9mmclip1.wav");
	}
	BOOL AddAmmo( CBaseEntity *pOther ) 
	{ 
		int iGive;

			iGive = AMMO_RPGCLIP_GIVE;


		if (pOther->GiveAmmo( iGive, "rockets", ROCKET_MAX_CARRY ) != -1)
		{
			EMIT_SOUND(ENT(pev), CHAN_ITEM, "items/9mmclip1.wav", 1, ATTN_NORM);
			return TRUE;
		}
		return FALSE;
	}
};
LINK_ENTITY_TO_CLASS( ammo_rpgclip, CRpgAmmo );

#endif
